/// <reference types="node" />
import { EventEmitter } from 'events';
import { Task, TaskMeta, TaskState } from './common';
import { BackendType } from './backends/interface';
import { MongodbBackendOptions } from './backends/mongodb';
import { BrokerType } from './brokers/interface';
import { AMQPBrokerOptions } from './brokers/amqp';
export interface Logger {
    info: (obj: object) => void;
    error: (obj: object) => void;
}
export interface ProcessFunc {
    (data: any, task?: Task): Promise<any>;
}
export interface ConsumerOptions {
    /**
     *  Broker type, default: AMQP
     */
    brokerType?: BrokerType;
    /**
     *  Broker options
     */
    brokerOptions?: AMQPBrokerOptions;
    /**
     *  Backend type, default: MongoDB
     */
    backendType?: BackendType;
    /**
     *  Backend options
     */
    backendOptions?: MongodbBackendOptions;
    /**
     *  Logger instance
     */
    logger?: Logger;
    /**
     *  Apm wrap, such as newrelic
     */
    processWrap?: (taskName: string, func: ProcessFunc) => ProcessFunc;
    /**
     *  Rre process hook
     */
    preProcess?: (task: Task) => void;
    /**
     *  Post process hook
     */
    postProcess?: (task: Task, state: TaskState, errorOrResult: any) => void;
    /**
     * Global concurrency for task, will be overrided by task, default: 256
     */
    globalConcurrency?: number;
}
export declare class Consumer extends EventEmitter {
    private options;
    private backend;
    private broker;
    constructor(config?: ConsumerOptions);
    private logSuccess(task, startTime, result);
    private logFail(task, startTime, e);
    registerTask(taskMeta: TaskMeta, processFunc: ProcessFunc): Promise<void>;
    close(): Promise<any>;
    checkHealth(): Promise<any>;
}
